@@ -47,6 +47,8 @@ class ONNXInit:
             path = os.getcwd()
         elif env == 'production':
             path = '/mnt/project'
+        else:
+            raise Exception('WTF!')
 
         parser.add_argument(
             '--model-path',
@@ -86,8 +88,19 @@ class ONNXInit:
         return model
 
 
+class ConfigInit:
+    @staticmethod
+    def register_parameters(parser):
+        parser
+        pass
+
+    @staticmethod
+    def init():
+        return Configurator()
+
 INIT_ARG_HANDLERS = {
     'onnx': ONNXInit,
+    'config': ConfigInit,
 }
 
 
@@ -229,9 +242,18 @@ def model_predict():
 
 @predict_blueprint.app_errorhandler(400)
 def handle_bad_request(e):
+    e
     return {'error': '无效的请求，请检查一下你的请求格式'}, 400
 
 
+class Configurator:
+
+    def limit_concurrency(self, n):
+        self
+        log.info(f'最大并发数限制为 {n}，超过并发限制的请求会返回 503 Service Unavailable')
+        current_app.config['OPENBAYES_SERVING_UVICORN_CONFIG']['limit_concurrency'] = n
+
+
 def make_app(predictor_cls):
     env = detect_env()
     if env == 'production':
@@ -255,6 +277,8 @@ def make_app(predictor_cls):
     log.info('Openbayes Serving 正在启动...')
 
     with app.app_context():
+        app.config['OPENBAYES_SERVING_UVICORN_CONFIG'] = {}
+
         lst = inspect.signature(predictor_cls).parameters.keys()
         clsname = getattr(predictor_cls, '__name__', repr(predictor_cls))
         args = {}
@@ -289,13 +313,12 @@ def make_app(predictor_cls):
 
     if env == 'local':
         log.info('检测到本地开发环境，开启调试模式')
-        app.register_blueprint(predict_blueprint, url_prefix='/')
         debug.install(app)
     elif env == 'gear':
         log.info('检测到 Openbayes 算力容器环境，开启调试模式')
+        prefix = None
 
         try:
-            prefix = None
             meta = requests.get('http://localhost:21999/gear-status').json()
             url = meta['links']['auxiliary']
             prefix = urlparse(url).path
@@ -306,10 +329,10 @@ def make_app(predictor_cls):
 
         app.register_blueprint(predict_blueprint, url_prefix=prefix)
         debug.install(app, url_prefix=prefix)
-
     elif env == 'production':
         log.info('检测到生产环境，关闭调试模式。请求的统计信息可以在 Openbayes 的控制台上查看。')
-        app.register_blueprint(predict_blueprint, url_prefix='/')
+
+    app.register_blueprint(predict_blueprint, url_prefix='/')
 
     return app
 
@@ -321,20 +344,19 @@ def run(predictor_cls):
     config = uvicorn.Config(app,
         host=options.host, port=options.port,
         log_level='info', interface='wsgi',
+        **app.config['OPENBAYES_SERVING_UVICORN_CONFIG']
     )
 
     logger = logging.getLogger('uvicorn')
     logger.handlers[:] = []
 
     patch_log_formatter(logging.getLogger('uvicorn.access'))
+    logging.getLogger('uvicorn.error').setLevel(logging.ERROR)
 
     server = uvicorn.Server(config=config)
-    log.info(f'Openbayes Serving 开始服务（{options.host}:{options.port}）')
+    log.info(f'Openbayes Serving 开始服务，本地访问地址：http://{options.host}:{options.port}')
     log.info('-' * 60)
     server.run()
 
 # def obt.serv.check(pattern, obj=<payload>):
 #     '''检查指定的对象是否满足 pattern，在 predict 处使用，默认检查 payload。要实现一个后门，用来给前端暴露这个 pattern 具体是什么。'''
-
-# def obt.serv.emit_event(filename, data):
-#     '''记录自定义事件。filename 是指事件的名字，约定使用文件名样子的名字。data 是实际的数据。如果 data 是 dict 或者 list，则会做 json encode；若为 string 或者 bytes，直接存储，否则报错。'''
